// --------------------------
// RP Core :: Commands
// --------------------------

// Startup command
function ServerCmdStartRP(%client)
{
	if ($RP::activated)
		return;
	if (!%client.isHost())
		return;
	// Please load everything already
	if (!%client.hasSpawnedOnce)
		return;
	messageAll('', "\c3Creating RP...");
	RPActivatePackages();
	BootRP();
}

// Shutdown command
function ServerCmdEndRP(%client)
{
	if (!$RP::activated)
		return;
	if (!%client.isHost())
		return;
	// Please load everything already
	if (!%client.hasSpawnedOnce)
		return;
	messageAll('', "\c3Removing RP...");
	ShutRP();
	RPDeactivatePackages();
}

// Joinging rp
function ServerCmdJoinRP(%client)
{
	if (!$RP::activated)
		return;
	if (!isObject(RP_MiniGame))
	{
		CreateMiniGame();
		// Users joining the RP
		%count = ClientGroup.getCount();
		for (%i = 0; %i < %count; %i++)
			ClientGroup.getObject(%i).EnterRPGame();
	}
	if (%client.isInRP())
		return;
	ServerCmdJoinMiniGame(%client, RP_MiniGame.getID());
	%client.EnterRPGame();
}

// Leaving rp
function ServerCmdLeaveRP(%client)
{
	if (!$RP::activated)
		return;
	if (!%client.isSuperAdmin)
		return;
	if (!%client.isInRP())
		return;
	ServerCmdLeaveMiniGame(%client);
}

// Package

if (isPackage(RP_Core_Commands))
	deactivatePackage(RP_Core_Commands);

package RP_Core_Commands
{
	// Fetch client commands and messages
	function ServerCmdMessageSent(%client, %text)
	{
		if (!%client.isInRP())
		{
			Parent::ServerCmdMessageSent(%client, %text);
			return;
		}
		// Activates a datablock
		if (isObject(%client.activateDatablock))
		{
			// Exit
			if (%text $= "0")
			{
				%client.StopActivate();
				messageClient(%client, '', "\c6Thank you come again.");
				return;
			}
			
			// Info brick
			if (%client.activateDatablock.isInfoBrick)
			{
				if (!%client.activateDatablock.parseData(%client, %text))
					messageClient(%client, '', "\c3Invalid command");
			}
			// Just clear some variables if you happen to get here
			else
			{
				%client.StopActivate();
				commandToClient(%client, 'centerPrint', "\c2This clickable brick is broken.", 2);
			}
			%client.displayInfo();
			return;
		}
		// Handle old command
		if (getSubStr(%text, 0, 1) $= "^" && getSubStr(%text, 1, 2) !$= "^" && getSubStr(%text, 1, 2) !$= "." && getSubStr(%text, 1, 2) !$= "_")
		{
			messageClient(%client, '', "\c6This is not CityRP with the \c3^\c6 command. Use \c3/\c6 instead.");
			return;
		}
		// All-Caps filter
		if ($RP::pref::server::capsFilter && strlen(%text) > 10 && strCmp(strUpr(%text), %text) == 0)
		{
			messageClient(%client, '', "\c6Please be friendly to Blockheads.");
			return;
		}
		
		// Local chat
		if ($RP::pref::server::localChat && isObject(%client.player))
		{
			%heard = 0;
			// Amount of players in the area
			InitContainerRadiusSearch(%client.player.getPosition(), $RP::pref::server::visibleDistance, $TypeMasks::PlayerObjectType);
			while((%search = ContainerSearchNext()) != 0)
				%heard++;

			%color = %heard > 1 ? "\c2" : "\c0";
			
			// Send the message
			InitContainerRadiusSearch(%client.player.getPosition(), $RP::pref::server::visibleDistance, $TypeMasks::PlayerObjectType);
			while((%search = ContainerSearchNext()) != 0)
				messageClient(%search.client, 'chatMessage', '%1[Local] \c7%2\c3%3\c7%4\c6: %5', %color, %client.clanPrefix, %client.name, %client.clanSuffix, %text);
		}
		else
		{
			Parent::ServerCmdMessageSent(%client, %text);
		}
	}
	
	// Team messages
	function ServerCmdTeamMessageSent(%client, %text)
	{
		if (isObject(RP_MiniGame) && $RP::pref::server::localChat && isObject(%client.player) && %client.isInRP())
		{
			messageAll('', '\c7%1\c3%2\c7%3\c6: %4', %client.clanPrefix, %client.name, %client.clanSuffix, %text);
		}
		else
		{
			Parent::ServerCmdTeamMessageSent(%client, %text);
		}
	}
};
RPRegPackage(RP_Core_Commands);

// Update the user stats
function ServerCmdUpdateInfo(%client)
{
	if (!$RP::activated)
		return;
	if (!%client.isInRP())
		return;
	%client.updateInfo();
}

// Toggle the big letters function
function ServerCmdToggleCapsFilter(%client)
{
	if (!$RP::activated)
		return;
	if (!%client.isInRP())
		return;
	// Super Admins only
	if (!%client.isSuperAdmin)
		return;
	$RP::pref::server::capsFilter = !$RP::pref::server::capsFilter;
	messageAll('', '\c6All-Caps Filter is now \c3%1\c6.', ($RP::pref::server::capsFilter ? "active" : "inactive"));
}

// Toggle report
function ServerCmdToggleReport(%client)
{
	if (!$RP::activated)
		return;
	if (!%client.isInRP())
		return;
	// Super Admins only
	if (!%client.isSuperAdmin)
		return;
	$RP::pref::server::report = !$RP::pref::server::report;
	messageAll('', '\c6The report system is now \c3%1\c6.', ($RP::pref::server::report ? "active" : "inactive"));
}

// Toggle suggestion
function ServerCmdToggleSuggest(%client)
{
	if (!$RP::activated)
		return;
	if (!%client.isInRP())
		return;
	// Super Admins only
	if (!%client.isSuperAdmin)
		return;
	$RP::pref::server::suggest = !$RP::pref::server::suggest;
	messageAll('', '\c6Suggestions are now \c3%1\c6.', ($RP::pref::server::suggest ? "active" : "inactive"));
}

// Toggle local chat
function ServerCmdToggleLocalChat(%client)
{
	if (!$RP::activated)
		return;
	if (!%client.isInRP())
		return;
	// Super Admins only
	if (!%client.isSuperAdmin)
		return;
	$RP::pref::server::localChat = !$RP::pref::server::localChat;
	messageAll('', '\c6Local chat is now \c3%1\c6.', ($RP::pref::server::localChat ? "active" : "inactive"));
}

// Set local chat range
function ServerCmdSetLocalChatRange(%client, %text)
{
	if (!$RP::activated)
		return;
	if (!%client.isInRP())
		return;
	// Super Admins only
	if (!%client.isSuperAdmin)
		return;
	$RP::pref::server::visibleDistance = (mFloor(%text) >= 10) ? mFloor(%text) : 10;
	// Update distance for each player
	for (%i = 0; %i < ClientGroup.getCount(); %i++)
	{
		%client = ClientGroup.getObject(%i);
		if (isObject(%client.player))
			%client.player.setShapeNameDistance($RP::pref::server::visibleDistance);
	}
	messageAll('', '\c6Local chat range is now \c3%1\c6.', $RP::pref::server::visibleDistance);
}

// Admin chat
function ServerCmdAdmin(%client, %v0, %v1, %v2, %v3, %v4, %v5, %v6, %v7, %v8, %v9, %v10, %v11, %v12, %v13, %v14, %v15, %v16, %v17, %v18, %v19, %v20)
{
	if (!$RP::activated)
		return;
	if (!%client.isInRP())
		return;
	if (!%client.isAdmin)
		return;
	// Prepare text
	%text = trim(%v0 SPC %v1 SPC %v2 SPC %v3 SPC %v4 SPC %v5 SPC %v6 SPC %v7 SPC %v8 SPC %v9 SPC %v10 SPC %v11 SPC %v12 SPC %v13 SPC %v14 SPC %v15 SPC %v16 SPC %v17 SPC %v18 SPC %v19 SPC %v20);
	
	// Empty text
	if (%text $= "")
		return;
	
	for (%i = 0; %i < ClientGroup.getCount(); %i++)
	{
		%target = ClientGroup.getObject(%i);
		if (%target.isAdmin)
			messageClient(%target, 'chatMessage', '\c3[\c0Admin\c3] %1: \c7%2', %client.name, %text);
	}
}

// Report
function ServerCmdReport(%client, %v0, %v1, %v2, %v3, %v4, %v5, %v6, %v7, %v8, %v9, %v10, %v11, %v12, %v13, %v14, %v15, %v16, %v17, %v18, %v19, %v20)
{
	if (!$RP::activated)
		return;
	if (!%client.isInRP())
		return;
	if (!$RP::pref::server::report)
	{
		messageClient(%client, '', "\c6The report system is currently inactive.");
		return;
	}
	
	// Prepare text
	%text = trim(%v0 SPC %v1 SPC %v2 SPC %v3 SPC %v4 SPC %v5 SPC %v6 SPC %v7 SPC %v8 SPC %v9 SPC %v10 SPC %v11 SPC %v12 SPC %v13 SPC %v14 SPC %v15 SPC %v16 SPC %v17 SPC %v18 SPC %v19 SPC %v20);
	// Empty text
	if (%text $= "")
	{
		messageClient(%client, '', "\c6Please use the following format:\c3 /report <\c0insert text here\c3>");
		messageClient(%client, '', "\c6Example:\c3 /report I can't plant bricks.");
		return;
	}
	
	// Add name, id and the report itself
	if (AppendToFile("report", %client.name @ ", " @ %client.getSaveKey() SPC "::" SPC %text))
		messageClient(%client, '', "\c6Report sent successfully.");
}

// Suggest
function ServerCmdSuggest(%client, %v0, %v1, %v2, %v3, %v4, %v5, %v6, %v7, %v8, %v9, %v10, %v11, %v12, %v13, %v14, %v15, %v16, %v17, %v18, %v19, %v20)
{
	if (!$RP::activated)
		return;
	if (!%client.isInRP())
		return;
	if (!$RP::pref::server::suggest)
	{
		messageClient(%client, '', "\c6Suggestions are currently inactive.");
		return;
	}
	
	// Prepare text
	%text = trim(%v0 SPC %v1 SPC %v2 SPC %v3 SPC %v4 SPC %v5 SPC %v6 SPC %v7 SPC %v8 SPC %v9 SPC %v10 SPC %v11 SPC %v12 SPC %v13 SPC %v14 SPC %v15 SPC %v16 SPC %v17 SPC %v18 SPC %v19 SPC %v20);
	// Empty text
	if (%text $= "")
	{
		messageClient(%client, '', "\c6Please use the following format:\c3 /suggest <\c0insert text here\c3>");
		messageClient(%client, '', "\c6Example:\c3 /suggest I would like to have a beach.");
		return;
	}
	
	// Add name, id and the suggestion itself
	if (AppendToFile("suggestions", %client.name @ ", " @ %client.getSaveKey() SPC "::" SPC %text))
		messageClient(%client, '', "\Suggestion sent successfully.");
}

// Get stats
function ServerCmdStats(%client, %arg0, %arg1, %arg2, %arg3)
{
	if (!$RP::activated)
		return;
	if (!%client.isInRP())
		return;
	// Closed
	if (!$RP::pref::server::stats)
		return false;
	
	if (!%client.isSuperAdmin)
	{
		%client.displayStats(%client.getSaveKey());
		return false;
	}
	
	%name = trim(%arg0 SPC %arg1 SPC %arg2 SPC %arg3);
	%target = (%name $= "") ? %client : findClientByText(%name);
	
	if (!isObject(%target))
	{
		// Is numeric
		if (%arg0 $= atoi(%arg0))
			%ID = %arg0;
		// It is you
		else
			%ID = %client.getSaveKey();
	}
	else
		%ID = %target.getSaveKey();
	
	%client.displayStats(%ID);
	return true;
	%haveGUI = %client.haveRPGUI("Core");
	
	// User exists in database
	if (RPDB.existKey[%ID])
	{
		// GUI controls
		if (%haveGUI)
		{
			for (%i = 1; %i <= $RP::stats::amount; %i++)
			{
				%var = RP_GetStatVar(%i, %ID);
				%stats = (%stats $= "") ? %row : %stats NL %var;
			}
			commandToClient(%client, 'DisplayRPStats', %stats);
		}
		else
		{
			for (%i = 1; %i <= $RP::stats::amount; %i++)
			{
				%var = RP_GetStatVar(%i, %ID);
				%stats = %stats @ "\c6" @ $RP::stats::name[%i] @ ": \c3" @ $RP::stats::prefix[%i] @ %var @ $RP::stats::suffix[%i] @ " ";
				// New line
				if ((%i-1) % 4 == 3)
				{
					messageClient(%client, '', %stats);
					%stats = "";
				}
				else if (%i == $RP::stats::amount)
				{
					messageClient(%client, '', %stats);
				}
			}
		}
		return true;
	}
	
	messageClient(%client, '', "\c6User does not exist.");
	return false;
}

///////////////////////
// Advanced commands //
///////////////////////

// Grant experience
function ServerCmdGrantExp(%client, %amount, %n0, %n1, %n2, %n3)
{
	if (!$RP::activated)
		return;
	if (!%client.isInRP())
		return;
	if (!%client.CheckPrefTrust($RP::pref::server::rankAdvanceCommands))
		return;
	
	%amount = mFloor(%amount);
	if (%amount <= 0)
	{
		messageClient(%client, '', "\c6Invalid amount.");
		return;
	}
	
	%name = trim(%n0 SPC %n1 SPC %n2 SPC %n3);
	%target = (%name $= "") ? %client : findClientByText(%name);
	
	if (!isObject(%target))
	{
		messageClient(%client, '', "\c6Invalid player.");
		return;
	}
	
	// Not enough rank but not yourself
	if (%target != %client && %target.GetPrefTrust() <= %client.GetPrefTrust())
	{
		messageClient(%client, '', '\c3%1\c6 has higher rank than you. Unable to grant experience.', %target.name);
		return;
	}
	
	RPDB.set(%target.getSaveKey(), "exp", RPDB.get(%target.getSaveKey(), "exp") + %amount);
	
	messageClient(%client, '', '\c6You granted \c3%1\c6 experience to \c3%2\c6.', %amount, %target.name);
	messageClient(%target, '', '\c3%1\c6 granted you \c3%2\c6 experience.', %client.name, %amount);
	
	%target.displayInfo();
}

// Deduct experience
function ServerCmdDeductExp(%client, %amount, %n0, %n1, %n2, %n3)
{
	if (!$RP::activated)
		return;
	if (!%client.isInRP())
		return;
	if (!%client.CheckPrefTrust($RP::pref::server::rankAdvanceCommands))
		return;
	
	%amount = mFloor(%amount);
	if (%amount <= 0)
	{
		messageClient(%client, '', "\c6Invalid amount.");
		return;
	}
	
	%name = trim(%n0 SPC %n1 SPC %n2 SPC %n3);
	%target = (%name $= "") ? %client : findClientByText(%name);
	
	if (!isObject(%target))
	{
		messageClient(%client, '', "\c6Invalid player.");
		return;
	}
	
	// Not enough rank but not yourself
	if (%target != %client && %target.GetPrefTrust() <= %client.GetPrefTrust())
	{
		messageClient(%client, '', '\c3%1\c6 has higher rank than you. Unable to deduct experience.', %target.name);
		return;
	}
	
	RPDB.set(%target.getSaveKey(), "exp", RPDB.get(%target.getSaveKey(), "exp") - %amount);
	
	messageClient(%client, '', '\c6You deducted \c3%1\c6 experience from \c3%2\c6.', %amount, %target.name);
	messageClient(%target, '', '\c3%1\c6 deducted \c3%2\c6 experience from you.', %client.name, %amount);
	
	%target.displayInfo();
}

// Clear everything
function ServerCmdClearPlayer(%client, %n0, %n1, %n2, %n3)
{
	if (!$RP::activated)
		return;
	if (!%client.isInRP())
		return;
	if (!%client.CheckPrefTrust($RP::pref::server::rankAdvanceCommands))
		return;
	
	%name = trim(%n0 SPC %n1 SPC %n2 SPC %n3);
	%target = (%name $= "") ? %client : findClientByText(%name);
	
	if (!isObject(%target))
	{
		messageClient(%client, '', "\c6Invalid player.");
		return;
	}
	
	// Not enough rank but not yourself
	if (%target != %client && %target.GetPrefTrust() <= %client.GetPrefTrust())
	{
		messageClient(%client, '', '\c3%1\c6 has higher rank than you. Unable to clear user data.', %target.name);
		return;
	}
	
	%target.RPData.clear();
	
	messageClient(%client, '', '\c6You cleared \c3%1\c6\'s user data.', %target.name);
	messageClient(%target, '', '\c3%1\c6 cleared your user data..', %client.name);
	
	// Respawn player
	if (isObject(%target.player))
		%target.player.delete();
	%target.spawnPlayer();
	
	%target.displayInfo();
}