// --------------------------
// RP Core :: User
// --------------------------

if (isPackage(RP_Core_User))
	deactivatePackage(RP_Core_User);

package RP_Core_User
{
	// Player entering a game
	function GameConnection::onClientEnterGame(%client)
	{
		Parent::onClientEnterGame(%client);
		
		if (!%client.isInRP())
			%client.EnterRPGame();
	}
	
	// Player leaving the game
	function GameConnection::onClientLeaveGame(%client)
	{
		if (%client.isInRP())
			%client.LeaveRPGame();
		
		Parent::onClientLeaveGame(%client);
	}
	
	// Spawn player
	function GameConnection::spawnPlayer(%client)
	{
		// Angus ghost
		if (isObject(%client.player))
			%client.player.delete();
		Parent::spawnPlayer(%client);
		
		if (%client.isInRP())
			%client.schedule(50, FinishRPSpawning);
	}
	
	// When the players gets killed
	function GameConnection::onDeath(%client, %killerPlayer, %killer, %damageType, %damageLoc)
	{
		Parent::onDeath(%client, %killerPlayer, %killer, %damageType, %damageLoc);
		
		// Loose tools
		if (isObject(%client) && %client.isInRP())
			%client.RPData.value["tools"] = "";
	}
	
	function GameConnection::setControlObject(%client, %obj)
	{
		Parent::setControlObject(%client, %obj);
		if (%client.player == %obj && %client.isInRP())
			commandToClient(%player.client, 'PlayGui_CreateToolHud', $RP::pref::user::amountTools);
	}
	
	// Change datablock
	function Player::changeDataBlock(%player, %datablock, %client)
	{
		if (isObject(RP_MiniGame) && %client.isInRP() && isObject(%client.RP_PlayerDatablock))
		{
			commandToClient(%player.client, 'PlayGui_CreateToolHud', %client.RP_PlayerDatablock.maxTools);
			// Force player datablock
			Parent::changeDataBlock(%player, %client.RP_PlayerDatablock, %client);
			// And remove it
			%client.RP_PlayerDatablock = "";
		}
		else if (isObject(RP_MiniGame) && %client.isInRP() && isObject($RP::pref::user::playerModel))
		{
			commandToClient(%player.client, 'PlayGui_CreateToolHud', $RP::pref::user::playerModel.maxTools);
			// Force player datablock
			Parent::changeDataBlock(%player, $RP::pref::user::playerModel, %client);
		}
		else
		{
			// Ordinary
			Parent::changeDataBlock(%player, %datablock, %client);
		}
	}
	
	// Evade scaling
	function Player::setScale(%player, %scale)
	{
		%client = %player.client;
		if (%client.isInRP() && !$RP::pref::user::playerScaling)
			%scale = $RP::pref::user::playerScale;
		Parent::setScale(%player, %scale);
	}
	
	// Activate stuffs
	function Player::activateStuff(%player)
	{
		Parent::activateStuff(%player);
		
		// Retreive target
		%obj = %player.GetObjectFromPOW($TypeMasks::fxBrickObjectType);
		
		if (!isObject(%obj))
			return 0;
		
		%client = %player.client;
		if (!%client.isInRP())
			return 0;
		// Avoid player press too many times
		if (!%client.CheckLastAction(%obj))
			return 0;
		
		// Brick
		if (%obj.getType() & $TypeMasks::fxBrickObjectType)
		{
			%datablock = %obj.getDataBlock();
			
			// Info brick
			if (%datablock.isInfoBrick)
			{
				%mods = %client.RPGUIMods;
				%mod = strlwr(%datablock.mod);
				%cmd = %datablock.guiCmd;
				%name = %datablock.getName();
				// GUI
				if (strlen(%cmd) != 0 && isFunction(%name, "ConstructGUI") && %client.HaveRPGUI(%mod))
				{
					%datablock.ConstructGUI(%client);
					commandToClient(%client, 'DisplayRPGUI', %cmd);
				}
				// Command
				else if (isFunction(%name, "Activate"))
				{
					if (%datablock.Activate(%client, %obj) != false)
					{
						messageClient(%client, '', "\c30. \c6Exit");
						%client.ThrowLastActive();
					}
				}
				// Invalid
				else
				{
					commandToClient(%client, 'centerPrint', "\c2Invalid clickable brick.", 2);
				}
				%client.lastAction = $sim::time;
			}
		}
		return 1;
	}
	
	function Armor::onNewDatablock(%armor, %player)
	{
		Parent::onNewDatablock(%armor, %player);
		%client = %player.client;
		if (!isObject(%client))
			return;
		
		commandToClient(%client, 'PlayGui_CreateToolHud', %armor.maxTools);
		for (%i = 0; %i < %armor.maxTools; %i++)
		{
			%tool = %player.tool[%i];
			messageClient(%client, 'MsgItemPickup', "", %i, (isObject(%tool) ? %tool.getID() : 0), 1);
		}
	}
};
RPRegPackage(RP_Core_User);

// Entering the RP
function GameConnection::EnterRPGame(%client)
{
	%key = %client.getSaveKey();
	if (RPDB.addKey(%key))
	{
		%data = RPDB.makeOnline(%key);
		if (isObject(%data))
			%client.RPData = %data;
	}
	
	// Update player name
	%name = ($Server::serverType $= "Internet") ? %client.name : %client.LANName;
	RPDB.set(%key, "name", %name);

	AddMember(%client); // Minigame
	%client.minigame = RP_MiniGame;
	
	commandToClient(%client, 'ControlRPGUI');
	
	// Check for GUI
	%client.SendModToGUI("core");
	
	%client.schedule(50, FinishRPJoining);
}

// Leaving the RP
function GameConnection::LeaveRPGame(%client)
{
	%player = %client.player;
	if (isObject(%player))
	{
		%tools = "";
		%db = %player.getDatablock();
		// Save equipments
		for (%i = 0; %i < %db.maxTools; %i++)
		{
			%tools = (%tools $= "") ? %player.tool[%i].uiName : %tools TAB %player.tool[%i].uiName;
		}
		%client.RPData.value["tools"] = %tools;
		%player.changeDataBlock(PlayerStandardArmor, %client);
	}
	// This is only for non-dedicated servers
	if (isObject(RPDB))
	{
		// Avoid multiple players
		%amount = 0;
		for (%i = 0; %i < ClientGroup.getCount(); %i++)
		{
			%target = ClientGroup.getObject(%i);
			if (%target.RPData == %client.RPData)
				%amount++;
		}
		if (%amount <= 1)
			RPDB.makeOffline(%client.getSaveKey());
	}
	// Remove member if minigame exists(As it should do)
	RemoveMember(%client);
	// Clear RP data
	%client.minigame = "";
	%client.RPData = "";
	// Remove the bottomprint
	commandToClient(%client, 'bottomPrint', "", 1);
}

// The finish when joining the server
function GameConnection::FinishRPJoining(%client)
{
	if (!isObject(%client))
		return;
	
	%client.FinishRPSpawning();
	
	commandToClient(%client, 'DisplayRPGUI', "Stats", 1);
}

// The finish when spawning the server
function GameConnection::FinishRPSpawning(%client, %noRespawn)
{
	if (!isObject(%client))
		return;
	
	%client.updateInfo();
	
	%player = %client.player;
	
	if (isObject(%player))
	{
		%player.RP_giveDefaultEquipment();
		
		// Make some player names and standard color
		%player.setShapeNameDistance($RP::pref::server::visibleDistance);
		%player.setShapeNameColor("1 1 1 1");
	}
}

// Display additional information to player
function GameConnection::displayInfo(%client)
{
	if (!isObject(%client))
		return;
	
	%gui = %client.haveRPGUI("core");
	
	$RP::stats::info[$RP::stats::var_i["exp"]] = $RP::pref::user::displayExp;
	
	if (%gui || (!$RP::pref::user::displayBottomPrint))
		return;
	
	// Initialize variables
	%info = "<font:palatino linotype:18>";
	
	// Go through list
	for (%i = 1; %i <= $RP::stats::amount; %i++)
	{
		if (!$RP::stats::info[%i])
			continue;
		%var = RP_GetStatVarFromData2(%i, %client.RPData);
		// Hidden value
		if ($RP::stats::hide[%i] && (%var == 0 || %var $= ""))
			continue;
		if (%var == atoi(%var))
			%var = getMetricPrefix(%var);
		%info = %info @ "\c6" @ $RP::stats::name[%i] @ ": \c3" @ $RP::stats::prefix[%i] @ %var @ $RP::stats::suffix[%i] @ " ";
		%infos = (%infos $= "") ? %var : %infos NL %var;
	}
	
	if (%gui)
		commandToClient(%client, 'UpdateRPStats', %infos);
	else
		commandToClient(%client, 'bottomPrint', %info, $RP::pref::user::updateInterval + 1);
}

// Display info each second
function GameConnection::updateInfo(%client)
{
	if (!$RP::activated)
		return;
	if (!isObject(%client))
		return;
	if (!%client.isInRP())
		return;
	if (isEventPending(%client.displayTime))
		Cancel(%client.displayTime);
	
	%client.displayInfo();
	
	// Only when no gui is found
	if (!%client.haveRPGUI("core"))
		%client.displayTime = %client.schedule(1000 * $RP::pref::user::updateInterval, updateInfo);
	else
		commandToClient(%client, 'bottomPrint', "", 1);
	
	return;
}

// Display stats
function GameConnection::displayStats(%client, %ID)
{
	if (!RPDB.existKey[%ID])
		return;
	
	// GUI controls
	if (%client.haveRPGUI("Core"))
	{
		for (%i = 1; %i <= $RP::stats::amount; %i++)
		{
			%var = RP_GetStatVar(%i, %ID);
			%stats = (%stats $= "") ? %row : %stats NL %var;
		}
		commandToClient(%client, 'DisplayRPStats', %stats);
	}
	else
	{
		for (%i = 1; %i <= $RP::stats::amount; %i++)
		{
			%var = RP_GetStatVar(%i, %ID);
			%stats = %stats @ "\c6" @ $RP::stats::name[%i] @ ": \c3" @ $RP::stats::prefix[%i] @ %var @ $RP::stats::suffix[%i] @ " ";
			// New line
			if ((%i-1) % 4 == 3)
			{
				messageClient(%client, '', %stats);
				%stats = "";
			}
			else if (%i == $RP::stats::amount)
			{
				messageClient(%client, '', %stats);
			}
		}
	}
}

// Check what sort of trust
function GameConnection::CheckPrefTrust(%client, %rank)
{
	switch (%rank)
	{
		case 0: // No one
			return false;
		case 1: // But host
			return %client.isHost();
		case 2: // But Super Admins
			return %client.isSuperAdmin;
		case 3: // But Admins
			return %client.isAdmin;
		case 4: // Anyone
			return true;
		default: // Value is wrong, go to default
			return %client.isSuperAdmin;
	}
}

// Get rank level
function GameConnection::GetPrefTrust(%client)
{
	switch (true)
	{
		case %client.isHost(): // Host
			return 1;
		case %client.isSuperAdmin: // Super Admins
			return 2;
		case %client.isAdmin: // Admins
			return 3;
		default: // Anyone
			return 4;
	}
}

// Give default equipments
function Player::RP_giveDefaultEquipment(%player)
{
	%client = %player.client;
	if (!isObject(%client))
		return;
	if (!%client.isInRP())
		return;
	// Choose source tools
	%tools = (%client.RPData.value["tools"] $= "") ? "Hammer " TAB "Wrench" TAB "Printer" : %client.RPData.value["tools"];
	
	%player.addTools(%tools);
	
	%client.RPData.value["tools"] = %tools;
}

// Add tools
function Player::addTools(%player, %tools)
{
	// Create talbe if not created
	if($UiNameTableCreated == false)
		createUiNameTable();
	
	%client = %player.client;
	
	%a = 0;
	%count = getFieldCount(%tools);
	// Go throught tools
	for (%i = 0; %a < %player.getDatablock().maxTools; %i++)
	{
		if (%i < %count)
		{
			// Add tool
			%toolID = $uiNameTable_Items[getField(%tools, %i)];
			if (!isObject(%toolID))
				continue;
			%player.tool[%a] = %toolID;
			messageClient(%client, 'MsgItemPickup', "", %a, %toolID);
		}
		else
		{
			// Reset tool
			%player.tool[%a] = "";
			messageClient(%client, 'MsgItemPickup', "", %a, 0);
		}
		%a++;
	}
}
	
// Get object from point of view
function Player::GetObjectFromPOW(%player, %mask)
{
	%mask = %mask || $TypeMasks::all;
	%point = %player.getEyePoint();
	%obj = containerRayCast(%point, vectorAdd(vectorScale(vectorNormalize(%player.getEyeVector()), 4), %point), %mask, %player);
	return firstWord(%obj);
}

// Check if can do an action
function GameConnection::CheckLastAction(%client, %obj)
{
	// If passed last action time
	return !(%client.lastAction > $sim::time - $RP::pref::user::lastActionTimer);
}

// Updates the last thing done on a clickable brick
function GameConnection::ThrowLastActive(%client)
{
	if (!isObject(%client))
		return;
	// Remove old one
	if (isEventPending(%client.activateSchedule))
	{
		Cancel(%client.activateSchedule);
		%client.activateSchedule = 0;
	}
	%client.activateSchedule = %client.schedule(1000 * $RP::pref::user::stopActivate, stopActivate);
}

// Remove clickable function
function GameConnection::StopActivate(%client)
{
	if (!isObject(%client))
		return;
	// Remove old one
	if (isEventPending(%client.activateSchedule))
	{
		Cancel(%client.activateSchedule);
		%client.activateSchedule = 0;
	}
	
	%client.target = 0;
	%client.activateDatablock = 0;
	%client.activateState = 0;
}

// Get a save key depending on game mode
function GameConnection::getSaveKey(%client)
{
	switch$ ($Server::ServerType)
	{
		case "SinglePlayer":
			return %client.LANname;
		case "LAN":
			return %client.getRawIP();
		case "Internet":
			return %client.bl_id;
		// Avoid return nothing
		default:
			return %client.name;
	}
}

// Is in RP minigame
function GameConnection::isInRP(%client)
{
	return isObject(RP_MiniGame) && (%client.minigame $= RP_MiniGame || %client.minigame $= RP_MiniGame.getID());
}

// Host function
function GameConnection::isHost(%client)
{
	return %client.bl_id == getNumKeyId() || %client.getRawIP() $= "local";
}